/*
 * Ray tracer model language parser.
 * The language is a subset of the rayshade model language.
 * This parser is mostly new because the rayshade parser was
 *  too complicated for my purposes.
 *
 *  Anselmo Lastra (lastra@cs.unc.edu) 9/17/00
 */

%{
#include "cinterf.h"
#include <stdio.h>
#include "raydefs.h"
extern int rayerror(char* s);
extern int raylex();

%}

/*
 * Declare the type of values in the grammar
 */

%union 
{
	int integer_type;
	double double_type;
	char *string_pointer;
}

/*
 * Token types: These are returned by the lexer
 */

%token <integer_type>	INT_NUM
%token <double_type>	DOUBLE_NUM
%token <string_pointer>	STRING_POINTER
%token <string_pointer>	tSTRING

%token	tAMBIENT tDIFFUSE tSPECULAR tREFLECTIVE tTRANSPARENT tINDEX
%token	tSPECPOW tREFLECT tTRANSP
%token	tPOINTLIGHT tLIGHT tPOINT tEXTENDED tEXTENDEDLIGHT
%token  tEYEP tFOV tLOOKP tUP tSCREEN
%token	tSCALE tTRANSLATE tROTATE
%token	tNAME tLIST tEND tOBJECT tSURFACE tAPPLYSURF
%token	tTEXTURE tIMAGE
%token	tBACKGROUND tSPHERE tPLANE tTRIANGLE
%token	tMINUS tPLUS
%type <double_type> Float

%%

/*
 * Production starts here
 */


/*rule1:	INT_NUM {$$=$1;}*/

Items	:
		| Items Item
		;

Item	: Ambient
		| Diffuse
		| Specular
		| Specpow
		| Reflect
		| Reflective
		| Transparent
		| Transp
		| Index
		| Pointlight
		| Eyep
		| Fov
		| Lookp
		| Up
		| Translate
		| Rotate
		| Scale
		| Background
		| Sphere
		| Plane
		| Triangle
		| Name
		| End
		| Object
		| Surface
		| Applysurf
		| Screen
		| Texture
		| Extendedlight
		;

Ambient	:tAMBIENT Float Float Float 
		{ Ambient($2, $3, $4);}
		;
Diffuse	: tDIFFUSE Float Float Float
		{ Diffuse($2, $3, $4);}
		;

Specular :tSPECULAR Float Float Float 
		{ Specular($2, $3, $4);}
		;

Specpow	: tSPECPOW Float
		  {Specpow($2); }
		;

Reflect :tREFLECT Float 
		{ Reflect($2); }
		;

Reflective :tREFLECTIVE Float 
		{ Reflective((int )$2); }
		;

Transparent : tTRANSPARENT Float
		{ Transparent((int )$2); }
		;

Transp : tTRANSP Float
		{ Transp($2); }
		;

Index	: tINDEX Float
		{ Index($2); }
		;
		
Pointlight : tPOINTLIGHT Float Float Float Float
		  { PointLight($2, $3, $4, $5);}
		| tLIGHT Float tPOINT Float Float Float
		  { PointLight($2, $4, $5, $6);}
		;

Extendedlight : tEXTENDEDLIGHT Float Float Float Float Float
		  { ExtendedLight($2, $3, $4, $5, $6);}
		| tLIGHT Float tEXTENDED Float Float Float Float
		  { ExtendedLight($2, $4, $5, $6, $7);}
		;

Eyep	:tEYEP Float Float Float 
		{ EyePoint($2, $3, $4);}
		;
Fov		:tFOV Float
		{ FieldOfView ($2);}
		;
Lookp	:tLOOKP Float Float Float 
		{ LookAt($2, $3, $4);}
		;
Up		:tUP Float Float Float 
		{ UpVector($2, $3, $4);}
		;

Screen	: tSCREEN Float Float
		  { Screen((int )$2, (int )$3); }
		;

Translate :tTRANSLATE Float Float Float 
		{ Translate($2, $3, $4);}
		;
Rotate	:tROTATE Float Float Float Float
		{ Rotate($2, $3, $4, $5);}
		;
Scale	:tSCALE Float Float Float 
		  { Scale($2, $3, $4);}
		;

Name	: tNAME tSTRING tLIST
		  { Name($2); }
		;
End		: tEND
		  { End(); }
		;
Object	: tOBJECT tSTRING
		  { Object($2); }
		| tOBJECT tSTRING tSTRING
		  { ObjectS($2, $3); }

		;
Surface	: tSURFACE tSTRING
		  { Surface($2); }
		;

Applysurf : tAPPLYSURF
		    { Applysurf(); }
		 ;

Texture	: tTEXTURE tIMAGE tSTRING
		   { Texture($3); }
		;

Background	:tBACKGROUND Float Float Float 
		  { Background($2, $3, $4);}
		;
Sphere	:tSPHERE Float Float Float Float
		  { Sphere($2, $3, $4, $5);}
		| tSPHERE tSTRING Float Float Float Float
		  { SphereS($2, $3, $4, $5, $6);}
		;
Plane	:tPLANE Float Float Float Float Float Float
		  { Plane($2, $3, $4, $5, $6, $7);}
		|tPLANE tSTRING Float Float Float Float Float Float
		  { PlaneS($2, $3, $4, $5, $6, $7, $8);}
		;
Triangle :tTRIANGLE Float Float Float Float Float Float Float Float Float
		  { Triangle($2, $3, $4, $5, $6, $7, $8, $9, $10);}
		| tTRIANGLE tSTRING Float Float Float Float Float Float Float Float Float
		  { TriangleS($2, $3, $4, $5, $6, $7, $8, $9, $10, $11);}
		| tTRIANGLE Float Float Float Float Float
		  Float Float Float Float Float
		  Float Float Float Float Float
		  { TriangleUV($2, $3, $4, $5, $6, $7, $8, $9, $10, $11,
				$12, $13, $14, $15, $16); }
		| tTRIANGLE tSTRING Float Float Float Float Float
		  Float Float Float Float Float
		  Float Float Float Float Float
		  { TriangleUVS($2, $3, $4, $5, $6, $7, $8, $9, $10, $11,
				$12, $13, $14, $15, $16, $17); }
		| tTRIANGLE Float Float Float Float Float Float Float Float Float 
		  Float Float Float Float Float Float Float Float Float
		  { TriangleN($2, $3, $4, $5, $6, $7, $8, $9, $10,
		         $11, $12, $13, $14, $15, $16, $17, $18, $19);}
		| tTRIANGLE tSTRING Float Float Float Float Float Float Float Float Float 
		  Float Float Float Float Float Float Float Float Float
		  { TriangleNS($2, $3, $4, $5, $6, $7, $8, $9, $10,
		         $11, $12, $13, $14, $15, $16, $17, $18, $19, $20);}
		;

Float	: DOUBLE_NUM
		| tMINUS DOUBLE_NUM
		{ $$ = -$2; }
		| tPLUS DOUBLE_NUM
		{ $$ = $2; }
		;

%%



/* programs */

/*
 * ???
 */
int
raywrap()
{
  return 1;
}

/*
 * Report an error situation discovered in a production
 *
 * This does not do anything since we report all error situations through
 * idl_global->err() operations
 */
int
rayerror(char *s)
{
    fprintf(stderr, "Syntax error: %s\n", s);
	return 0;
}
