
//
// Function prototypes for simple ray tracing model description
//
// I tried to make this so that it would work in plain 'C' (after some simple global
//  editing), but did not test it.
//
//	Anselmo Lastra (lastra@cs.unc.edu), 9/17/00


// Center of projection of camera
//

extern void EyePoint(double x, double y, double z);

// Look-at point for camera
//
extern void LookAt(double x, double y, double z);

// Up Vector
//

extern void UpVector(double x, double y, double z);

// FOV in degrees
//

extern void FieldOfView(double angle);

// Color of background
//

extern void Background(double r, double g, double b);


// Point light source with intensity and location
//

extern void PointLight(double intensity, double x, double y, double z);

extern void ExtendedLight(double intensity, double radius,
			  double x, double y, double z);

// ** Surface Properties

//A named surface description

extern void Surface(char *);

// The surface description which follows (not named)
// should become active immediately.

extern void Applysurf();

// Types of surface properties
//
extern void Ambient(double r, double g, double b);

extern void Diffuse(double r, double g, double b);

extern void Specular(double r, double g, double b);

// Declares surface as reflective
// If not reflective (0), don't shoot reflective rays
//
extern void Reflective(int reflect);

// Similar for transparent
//   You could add a function for transparency color
//   or intrinsic color of the surface

extern void Transparent(int trans);

// Index of refraction
//   Default should be 1.0

extern void Index(double index);

extern void Specpow(double);

extern void Reflect(double);

extern void Transp(double);


// ** Primitives

// A plane described by a point and a normal.
//

extern void Plane(double point_x, double point_y, double point_z,
  double normal_x, double normal_y, double normal_z);
extern void PlaneS(char *surface, double point_x, double point_y, double point_z,
  double normal_x, double normal_y, double normal_z);


// A sphere described by radius and center
//

extern void Sphere(double radius, double x, double y, double z);
extern void SphereS(char *surface, double radius, double x, double y, double z);

// Triangle, with and without surface name
//

extern void TriangleS(char *surface,
					 double v1x, double v1y, double v1z,
					 double v2x, double v2y, double v2z,
					 double v3x, double v3y, double v3z);

extern void Triangle(double v1x, double v1y, double v1z,
					 double v2x, double v2y, double v2z,
					 double v3x, double v3y, double v3z);

// Triangle with UV
//

extern void TriangleUVS(char *surface,
					   double v1x, double v1y, double v1z, double v1u, double v1v,
					   double v2x, double v2y, double v2z, double v2u, double v2v,
					   double v3x, double v3y, double v3z, double v3u, double v3v);

extern void TriangleUV(double v1x, double v1y, double v1z, double v1u, double v1v,
					   double v2x, double v2y, double v2z, double v2u, double v2v,
					   double v3x, double v3y, double v3z, double v3u, double v3v);

//
// Triangle with normals (not used for course)
//
extern void TriangleNS(char *surface,
					  double v1x, double v1y, double v1z,
					  double n1x, double n1y, double n1z,
					  double v2x, double v2y, double v2z,
					  double n2x, double n2y, double n2z,
					  double v3x, double v3y, double v3z,
					  double n3x, double n3y, double n3z);

extern void TriangleN(double v1x, double v1y, double v1z,
					  double n1x, double n1y, double n1z,
					  double v2x, double v2y, double v2z,
					  double n2x, double n2y, double n2z,
					  double v3x, double v3y, double v3z,
					  double n3x, double n3y, double n3z);

extern void Translate(double x, double y, double z);

/* Rotate angle around axis specified by x,y,z */
extern void Rotate(double x, double y, double z, double angle);

extern void Scale(double x, double y, double z);

// Name of an object to define

extern void Name(char *s);

// End of object definition

extern void End();


// An instantiation of a previously defined object

extern void Object(char *name);
extern void ObjectS(char *surface, char*name);

// Size of image to produce

extern void Screen(int x, int y);

extern void Texture(char *);
